import os.path as op
from django.core.files.storage import FileSystemStorage
from django.db import models
from django.utils import simplejson as json
from fsfield import FileStorageField


tmp_dir = op.join(op.dirname(__file__), "tmp")
storage = FileSystemStorage(tmp_dir)


class StorageModel(models.Model):

    normal_field = models.CharField(max_length=255)
    simple_file_field = FileStorageField(storage=storage)
    json_file_field = FileStorageField(load=json.load, dump=json.dump,
            storage=storage)
    custom_name_file_field = FileStorageField(filename="custom",
            storage=storage)
    default_storage_field = FileStorageField()



import os
import os.path as op
from django.db.models.signals import post_save
from django.dispatch import receiver
from django.core.files.storage import FileSystemStorage
from django.core.files import locks
from fsfield.core import model_instance_field_path, default_storage


class FileStorageFieldDescriptor(object):

    def __init__(self, name, filename, storage, default, load, dump):
        self.name = name
        self.filename = filename
        self.storage = storage
        self.default = default
        self.load = load
        self.dump = dump

    def path(self, obj):
        if obj.pk is None:
            raise AttributeError("you must save the object to the database "
                    "before accessing this field")
        return model_instance_field_path(obj, self.filename)

    def __get__(self, obj, type=None):
        if obj is None:
            return
        if self.name not in obj.__dict__:
            # Load data from disk
            path = self.path(obj)
            if not self.storage.exists(path):
                value = self.default
            else:
                # Open the file for reading and acquire a lock on it if
                # possible
                fp = self.storage.open(path, "r+b")
                if isinstance(self.storage, FileSystemStorage):
                    locks.lock(fp.file, locks.LOCK_EX)
                try:
                    # Read file content
                    if self.load is not None:
                        value = self.load(fp)
                    else:
                        value = fp.read().decode("utf8")
                finally:
                    # Release lock
                    if isinstance(self.storage, FileSystemStorage):
                        locks.unlock(fp.file)
                    # Close file
                    fp.close()
            obj.__dict__[self.name] = value
        return obj.__dict__[self.name]

    def __set__(self, obj, value):
        obj.__dict__[self.name] = value


class FileStorageField(object):
    """
    This field type stores string data on the disk, bypassing entirely the
    database.

    *storage* may be a :class:`~django.core.files.storage.Storage` subclass to
    customize where the files are stored. The default storage is used if this
    is left unspecified.
    
    You may specify callables in *load* and *dump* to alter the way data is
    loaded from and saved to disk::

        load(fp)
        dump(data, fp)

    Where ``fp`` is a file-like object returned by the storage system.

    *default* is the value returned when the file associated to the field
    doesn't exist.

    *filename* may be used to customize the name of the files corresponding to
    this field. The field name is used by default.
    """

    def __init__(self, storage=None, load=None, dump=None, default=None,
            filename=None):
        self.load = load
        self.dump = dump
        if storage is None:
            self.storage = default_storage()
        else:
            self.storage = storage
        self.default = default
        self.filename = filename

    def contribute_to_class(self, cls, name):
        if self.filename is not None:
            filename = self.filename
        else:
            filename = name

        # Create a post_save signal handler to write data to disk 
        @receiver(post_save, sender=cls, weak=False,
                dispatch_uid="fsfield_write_data_%s_%s_%s" % 
                (cls.__module__, cls.__name__, name))
        def write_data(sender, instance, created, raw, using, **kwargs):
            if (name not in instance.__dict__ or
                    instance.__dict__[name] is None):
                # Nothing to save
                return
            # Open the file for writing and acquire a lock on it if
            # possible
            path = model_instance_field_path(instance, filename)
            fs_storage = isinstance(self.storage, FileSystemStorage)
            if fs_storage:
                full_path = self.storage.path(path)
                directory = op.dirname(full_path)
                if not op.exists(directory):
                    try:
                        os.makedirs(directory)
                    except OSError, err:
                        # Another thread may have created the directory since
                        # the check
                        if err.errno == 17:
                            pass
            fp = self.storage.open(path, "wb")
            if fs_storage:
                locks.lock(fp.file, locks.LOCK_EX)
            # Write data
            try:
                value = instance.__dict__[name]
                if self.dump is None:
                    fp.write(value.encode("utf8"))
                else:
                    self.dump(value, fp)
            finally:
                # Release lock
                if fs_storage:
                    locks.unlock(fp.file)
                # Close file
                fp.close()

        # Install field descriptor
        descriptor = FileStorageFieldDescriptor(name, filename, self.storage,
                self.default, self.load, self.dump)
        setattr(cls, name, descriptor)



import os.path as op
import shutil
from django.conf import settings
from fsfield.tests.base import SettingsTestCase
from fsfield.tests.models import StorageModel
from fsfield.core import model_instance_field_path
from fsfield.tests.models import storage


tmp_dir = op.join(op.dirname(__file__), "tmp")


class FileStorageFieldTests(SettingsTestCase):

    def setUp(self):
        new_apps = settings.INSTALLED_APPS + ["fsfield.tests"]
        self.settings_manager.set(
            INSTALLED_APPS=new_apps,
        )
        if op.isdir(tmp_dir):
            shutil.rmtree(tmp_dir)

    def test_class_access(self):
        self.assertEqual(StorageModel.simple_file_field, None)

    def test_simple_field(self):
        obj = StorageModel.objects.create()
        self.assertEqual(obj.simple_file_field, None)
        obj.simple_file_field = "foo"
        obj.save()
        obj = StorageModel.objects.get(pk=obj.pk)
        self.assertEqual(obj.simple_file_field, "foo")

    def test_field_with_accessors(self):
        obj = StorageModel.objects.create()
        obj.json_file_field = {"foo": 12}
        obj.save()
        obj = StorageModel.objects.get(pk=obj.pk)
        self.assertEqual(obj.json_file_field, {"foo": 12})

    def test_errors(self):
        obj = StorageModel()
        self.assertRaises(AttributeError, getattr, obj, "simple_file_field")
    
    def test_custom_filename_field(self):
        obj = StorageModel.objects.create()
        obj.custom_name_file_field = "flap"
        obj.save()
        path = storage.path(model_instance_field_path(obj, "custom"))
        self.assertEqual(op.exists(path), True)
        self.assertEqual(open(path).read(), "flap")



from django.conf import settings
from django.core.management import call_command
from django.db.models import loading
from django.test import TestCase


NO_SETTING = ('!', None)


class TestSettingsManager(object):
    """
    A class which can modify some Django settings temporarily for a
    test and then revert them to their original values later.

    Automatically handles resyncing the DB if INSTALLED_APPS is
    modified.

    """
    def __init__(self):
        self._original_settings = {}

    def set(self, **kwargs):
        for k,v in kwargs.iteritems():
            self._original_settings.setdefault(k, getattr(settings, k,
                                                          NO_SETTING))
            setattr(settings, k, v)
        if 'INSTALLED_APPS' in kwargs:
            self.syncdb()

    def syncdb(self):
        loading.cache.loaded = False
        call_command('syncdb', verbosity=0)

    def revert(self):
        for k,v in self._original_settings.iteritems():
            if v == NO_SETTING:
                delattr(settings, k)
            else:
                setattr(settings, k, v)
        if 'INSTALLED_APPS' in self._original_settings:
            self.syncdb()
        self._original_settings = {}


class SettingsTestCase(TestCase):
    """
    A subclass of the Django TestCase with a settings_manager
    attribute which is an instance of TestSettingsManager.

    Comes with a tearDown() method that calls
    self.settings_manager.revert().

    """
    def __init__(self, *args, **kwargs):
        super(SettingsTestCase, self).__init__(*args, **kwargs)
        self.settings_manager = TestSettingsManager()
    
    def tearDown(self):
        self.settings_manager.revert()



# -*- coding: utf8 -*-
import os.path as op
from django.test import TestCase
from fsfield.core import path_hash, hashed_path


class CoreTests(TestCase):

    def test_path_hash(self):
        self.assertEqual(path_hash("abc"), "qZk_NkcGgWq6PiVxeFDCbJzQ2J0")
        self.assertEqual(path_hash("abcé"), "bTkvtRcwOXVMzUc-qEmUth6Z3OM")
        self.assertEqual(path_hash(1), "NWoZK3kTsExUV00Ywo1G5jlUKKs")

    def test_hashed_path(self):
        self.assertEqual(hashed_path(1, 3), 
                op.join("N", "W", "o", "NWoZK3kTsExUV00Ywo1G5jlUKKs"))



import os.path as op
from base64 import b64encode
import hashlib
from django.utils.encoding import force_unicode
from fsfield import settings


_default_storage = None


def path_hash(value):
    """
    Get the hash used to store something named *value* in the filesystem.
    """
    value_str = force_unicode(value).encode("utf8")
    digest = hashlib.sha1(value_str).digest()
    return b64encode(digest, "_-").rstrip("=")


def hashed_path(value, depth):
    """
    Get the hashed path corresponding to *value*, with *depth* levels of 
    sub-directories.
    """
    hashed = path_hash(value)
    components = [hashed[i] for i in range(depth)]
    components.append(hashed)
    return op.join(*components)
        

def field_path(app_label, model_name, pk, field_name):
    """
    Lower level version of :func:`model_instance_field_path`.
    """
    return op.join(
            app_label, 
            model_name, 
            hashed_path(pk, settings.PATHS_DEPTH), 
            field_name)


def model_instance_field_path(instance, field_name):
    """
    Get the path used to store the Django model *instance* field named
    *field_name*.

    Note that the returned path is a relative path, to get the absolute path,
    you must pass the value returned by this function to your storage's
    :meth:`~django.core.files.storage.Storage.path` method.
    """
    return field_path(
            instance._meta.app_label,
            instance._meta.object_name, 
            instance.pk,
            field_name)


def default_storage():
    """
    Get the default storage instance configured in settings.
    """
    global _default_storage
    if _default_storage is None:
        module, cls_name = settings.DEFAULT_STORAGE_CLASS.rsplit(".", 1)
        args, kwargs = settings.DEFAULT_STORAGE_ARGS
        cls = getattr(__import__(module, {}, {}, cls_name), cls_name)
        _default_storage = cls(*args, **kwargs)
    return _default_storage



from django.conf import settings


DEFAULT_STORAGE_CLASS = getattr(settings, "FSFIELD_DEFAULT_STORAGE_CLASS",
        "django.core.files.storage.FileSystemStorage")
DEFAULT_STORAGE_ARGS = getattr(settings, "FSFIELD_DEFAULT_STORAGE_ARGS", 
        ((), {}))
PATHS_DEPTH = getattr(settings, "FSFIELD_PATHS_DEPTH", 5)



#!/usr/bin/env python
from setuptools import setup, find_packages
import os

here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README.rst')).read()
NEWS = open(os.path.join(here, 'NEWS.txt')).read()


version = '0.5.3'

install_requires = [
    # List your project dependencies here.
    # For more details, see:
    # http://packages.python.org/distribute/setuptools.html#declaring-dependencies
]


setup(name='django-fsfield',
    version=version,
    description="Scalabe file storage fields for your Django models",
    long_description=README + '\n\n' + NEWS,
    classifiers=[
      # Get strings from http://pypi.python.org/pypi?%3Aaction=list_classifiers
    ],
    keywords='django file storage models',
    author='Luper Rouch',
    author_email='luper.rouch@gmail.com',
    url='',
    license='BSD License',
    packages=find_packages('src'),
    package_dir = {'': 'src'},
    include_package_data=True,
    zip_safe=False,
    install_requires=install_requires,
)
